# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import sys
from typing import Any, Callable, Dict, IO, Optional, TypeVar, Union, overload

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._event_sources_operations import (
    build_create_or_update_request,
    build_delete_request,
    build_get_request,
    build_list_by_environment_request,
    build_update_request,
)

if sys.version_info >= (3, 8):
    from typing import Literal  # pylint: disable=no-name-in-module, ungrouped-imports
else:
    from typing_extensions import Literal  # type: ignore  # pylint: disable=ungrouped-imports
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class EventSourcesOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.timeseriesinsights.aio.TimeSeriesInsightsClient`'s
        :attr:`event_sources` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @overload
    async def create_or_update(
        self,
        resource_group_name: str,
        environment_name: str,
        event_source_name: str,
        parameters: _models.EventSourceCreateOrUpdateParameters,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.EventSourceResource:
        """Create or update an event source under the specified environment.

        :param resource_group_name: Name of an Azure Resource group. Required.
        :type resource_group_name: str
        :param environment_name: The name of the Time Series Insights environment associated with the
         specified resource group. Required.
        :type environment_name: str
        :param event_source_name: Name of the event source. Required.
        :type event_source_name: str
        :param parameters: Parameters for creating an event source resource. Required.
        :type parameters: ~azure.mgmt.timeseriesinsights.models.EventSourceCreateOrUpdateParameters
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: EventSourceResource or the result of cls(response)
        :rtype: ~azure.mgmt.timeseriesinsights.models.EventSourceResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def create_or_update(
        self,
        resource_group_name: str,
        environment_name: str,
        event_source_name: str,
        parameters: IO,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.EventSourceResource:
        """Create or update an event source under the specified environment.

        :param resource_group_name: Name of an Azure Resource group. Required.
        :type resource_group_name: str
        :param environment_name: The name of the Time Series Insights environment associated with the
         specified resource group. Required.
        :type environment_name: str
        :param event_source_name: Name of the event source. Required.
        :type event_source_name: str
        :param parameters: Parameters for creating an event source resource. Required.
        :type parameters: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: EventSourceResource or the result of cls(response)
        :rtype: ~azure.mgmt.timeseriesinsights.models.EventSourceResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def create_or_update(
        self,
        resource_group_name: str,
        environment_name: str,
        event_source_name: str,
        parameters: Union[_models.EventSourceCreateOrUpdateParameters, IO],
        **kwargs: Any
    ) -> _models.EventSourceResource:
        """Create or update an event source under the specified environment.

        :param resource_group_name: Name of an Azure Resource group. Required.
        :type resource_group_name: str
        :param environment_name: The name of the Time Series Insights environment associated with the
         specified resource group. Required.
        :type environment_name: str
        :param event_source_name: Name of the event source. Required.
        :type event_source_name: str
        :param parameters: Parameters for creating an event source resource. Is either a model type or
         a IO type. Required.
        :type parameters: ~azure.mgmt.timeseriesinsights.models.EventSourceCreateOrUpdateParameters or
         IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: EventSourceResource or the result of cls(response)
        :rtype: ~azure.mgmt.timeseriesinsights.models.EventSourceResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version = kwargs.pop(
            "api_version", _params.pop("api-version", self._config.api_version)
        )  # type: Literal["2021-03-31-preview"]
        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.EventSourceResource]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(parameters, (IO, bytes)):
            _content = parameters
        else:
            _json = self._serialize.body(parameters, "EventSourceCreateOrUpdateParameters")

        request = build_create_or_update_request(
            resource_group_name=resource_group_name,
            environment_name=environment_name,
            event_source_name=event_source_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self.create_or_update.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if response.status_code == 200:
            deserialized = self._deserialize("EventSourceResource", pipeline_response)

        if response.status_code == 201:
            deserialized = self._deserialize("EventSourceResource", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    create_or_update.metadata = {"url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.TimeSeriesInsights/environments/{environmentName}/eventSources/{eventSourceName}"}  # type: ignore

    @distributed_trace_async
    async def get(
        self, resource_group_name: str, environment_name: str, event_source_name: str, **kwargs: Any
    ) -> _models.EventSourceResource:
        """Gets the event source with the specified name in the specified environment.

        :param resource_group_name: Name of an Azure Resource group. Required.
        :type resource_group_name: str
        :param environment_name: The name of the Time Series Insights environment associated with the
         specified resource group. Required.
        :type environment_name: str
        :param event_source_name: The name of the Time Series Insights event source associated with the
         specified environment. Required.
        :type event_source_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: EventSourceResource or the result of cls(response)
        :rtype: ~azure.mgmt.timeseriesinsights.models.EventSourceResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version = kwargs.pop(
            "api_version", _params.pop("api-version", self._config.api_version)
        )  # type: Literal["2021-03-31-preview"]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.EventSourceResource]

        request = build_get_request(
            resource_group_name=resource_group_name,
            environment_name=environment_name,
            event_source_name=event_source_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            template_url=self.get.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("EventSourceResource", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get.metadata = {"url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.TimeSeriesInsights/environments/{environmentName}/eventSources/{eventSourceName}"}  # type: ignore

    @overload
    async def update(
        self,
        resource_group_name: str,
        environment_name: str,
        event_source_name: str,
        event_source_update_parameters: _models.EventSourceUpdateParameters,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.EventSourceResource:
        """Updates the event source with the specified name in the specified subscription, resource group,
        and environment.

        :param resource_group_name: Name of an Azure Resource group. Required.
        :type resource_group_name: str
        :param environment_name: The name of the Time Series Insights environment associated with the
         specified resource group. Required.
        :type environment_name: str
        :param event_source_name: The name of the Time Series Insights event source associated with the
         specified environment. Required.
        :type event_source_name: str
        :param event_source_update_parameters: Request object that contains the updated information for
         the event source. Required.
        :type event_source_update_parameters:
         ~azure.mgmt.timeseriesinsights.models.EventSourceUpdateParameters
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: EventSourceResource or the result of cls(response)
        :rtype: ~azure.mgmt.timeseriesinsights.models.EventSourceResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def update(
        self,
        resource_group_name: str,
        environment_name: str,
        event_source_name: str,
        event_source_update_parameters: IO,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.EventSourceResource:
        """Updates the event source with the specified name in the specified subscription, resource group,
        and environment.

        :param resource_group_name: Name of an Azure Resource group. Required.
        :type resource_group_name: str
        :param environment_name: The name of the Time Series Insights environment associated with the
         specified resource group. Required.
        :type environment_name: str
        :param event_source_name: The name of the Time Series Insights event source associated with the
         specified environment. Required.
        :type event_source_name: str
        :param event_source_update_parameters: Request object that contains the updated information for
         the event source. Required.
        :type event_source_update_parameters: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: EventSourceResource or the result of cls(response)
        :rtype: ~azure.mgmt.timeseriesinsights.models.EventSourceResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def update(
        self,
        resource_group_name: str,
        environment_name: str,
        event_source_name: str,
        event_source_update_parameters: Union[_models.EventSourceUpdateParameters, IO],
        **kwargs: Any
    ) -> _models.EventSourceResource:
        """Updates the event source with the specified name in the specified subscription, resource group,
        and environment.

        :param resource_group_name: Name of an Azure Resource group. Required.
        :type resource_group_name: str
        :param environment_name: The name of the Time Series Insights environment associated with the
         specified resource group. Required.
        :type environment_name: str
        :param event_source_name: The name of the Time Series Insights event source associated with the
         specified environment. Required.
        :type event_source_name: str
        :param event_source_update_parameters: Request object that contains the updated information for
         the event source. Is either a model type or a IO type. Required.
        :type event_source_update_parameters:
         ~azure.mgmt.timeseriesinsights.models.EventSourceUpdateParameters or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: EventSourceResource or the result of cls(response)
        :rtype: ~azure.mgmt.timeseriesinsights.models.EventSourceResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version = kwargs.pop(
            "api_version", _params.pop("api-version", self._config.api_version)
        )  # type: Literal["2021-03-31-preview"]
        content_type = kwargs.pop("content_type", _headers.pop("Content-Type", None))  # type: Optional[str]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.EventSourceResource]

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(event_source_update_parameters, (IO, bytes)):
            _content = event_source_update_parameters
        else:
            _json = self._serialize.body(event_source_update_parameters, "EventSourceUpdateParameters")

        request = build_update_request(
            resource_group_name=resource_group_name,
            environment_name=environment_name,
            event_source_name=event_source_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self.update.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("EventSourceResource", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    update.metadata = {"url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.TimeSeriesInsights/environments/{environmentName}/eventSources/{eventSourceName}"}  # type: ignore

    @distributed_trace_async
    async def delete(  # pylint: disable=inconsistent-return-statements
        self, resource_group_name: str, environment_name: str, event_source_name: str, **kwargs: Any
    ) -> None:
        """Deletes the event source with the specified name in the specified subscription, resource group,
        and environment.

        :param resource_group_name: Name of an Azure Resource group. Required.
        :type resource_group_name: str
        :param environment_name: The name of the Time Series Insights environment associated with the
         specified resource group. Required.
        :type environment_name: str
        :param event_source_name: The name of the Time Series Insights event source associated with the
         specified environment. Required.
        :type event_source_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version = kwargs.pop(
            "api_version", _params.pop("api-version", self._config.api_version)
        )  # type: Literal["2021-03-31-preview"]
        cls = kwargs.pop("cls", None)  # type: ClsType[None]

        request = build_delete_request(
            resource_group_name=resource_group_name,
            environment_name=environment_name,
            event_source_name=event_source_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            template_url=self.delete.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    delete.metadata = {"url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.TimeSeriesInsights/environments/{environmentName}/eventSources/{eventSourceName}"}  # type: ignore

    @distributed_trace_async
    async def list_by_environment(
        self, resource_group_name: str, environment_name: str, **kwargs: Any
    ) -> _models.EventSourceListResponse:
        """Lists all the available event sources associated with the subscription and within the specified
        resource group and environment.

        :param resource_group_name: Name of an Azure Resource group. Required.
        :type resource_group_name: str
        :param environment_name: The name of the Time Series Insights environment associated with the
         specified resource group. Required.
        :type environment_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: EventSourceListResponse or the result of cls(response)
        :rtype: ~azure.mgmt.timeseriesinsights.models.EventSourceListResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version = kwargs.pop(
            "api_version", _params.pop("api-version", self._config.api_version)
        )  # type: Literal["2021-03-31-preview"]
        cls = kwargs.pop("cls", None)  # type: ClsType[_models.EventSourceListResponse]

        request = build_list_by_environment_request(
            resource_group_name=resource_group_name,
            environment_name=environment_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            template_url=self.list_by_environment.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("EventSourceListResponse", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_by_environment.metadata = {"url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.TimeSeriesInsights/environments/{environmentName}/eventSources"}  # type: ignore
